function G = gmatas_q2(theta0,n,d)

%Function to compute local identification condition for the AS(2007) model at
%a point theta0 under both determinacy and indeterminacy. 
%Inputs: theta0 - vector of model parameters in the order:
%[tau beta kappa psi1 psi2 rhoR rhog rhoz sigR sigg sigz,m_eta_r,m_eta_g,m_eta_a,sig_eta]';
%
% n - number of points for evaluating the integral
% d - (in)determinacy index. =1 for indeterminacy, =0 for determinacy 
%Observables order: 1) inflation, 2) output, 3) interest rate

%uses auxiliary files createcov_as.m, dsgesolv.m, gensys_mod.m, svdrr.m,
%lgwt.m

%The integral is approximated using Gauss-Legendre quadrature with n points

%The numerical derivative is computed using the symmetric difference quotient rule

%modification for computing the curve to keep step size constant along the curve:
if d==1
    load theta0asi
elseif d==0
    load theta0asd
end

%% Model solution

[TT,TC,TEPS,TETA,RC] = dsgesolv(theta0,1); 
nstate = size(TT,2); %number of state variables


if RC==[1;1] %if determinacy
    RR = [TEPS,zeros(nstate,1)];
    
 
    QQ=createcov_as(theta0); %covariance matrix, zeros for indeterminacy param's
    
elseif RC==[1;0] %if indeterminacy
    TETA=rref(TETA').'; %reduced column echelon form for TETA
    RR = [TEPS,TETA]; %note - to conform with Gauss/LS gensys
    QQ=createcov_as(theta0); %augmented covariance matrix
else
    G=[];
    disp('Nonexistence or numerical problems')
    return
end
%% Quadrature abscissae and weights
% [x,wg] = mp.GaussLegendre(n,-pi,pi);
% 
% x=double(x); %frequencies for approximating integral (convert to double from mp)
% wg=double(wg); %Gauss-Lengendre quadrature weights for each frequency (converted to double from mp)

% %Alternatively:
[x,wg] = lgwt(n,-pi,pi); 


%% Spectrum computation
%set up selection matrix
A=[1	0	0	0	0	0	0;
   0	1	0	0	0	0	0;
   0	0	1	0	0	0	0]; %selection matrix A

%the integral [-pi,pi] is approximated using n points

dimy=size(A,1); %number of observables
% sp_mat=zeros(dimy*n,dimy); %blank for spectra at all frequencies in omi
% for i=1:n
% exe=exp(-1i*x(i));
% %A(2,2)=(1-exe);
% mat1=(eye(nstate)-TT*exe)\eye(nstate); %inv(1-T1L)
% mat2=mat1';  %note that ' gives the conjugate transpose
% 
% sp_mat(((i-1)*dimy+1):(i*dimy),:)=A*mat1*RR*QQ*RR'*mat2*A'/(2*pi);
% %A(2,2)=1;
% end
%% Numerical derivative
nsp_mat=zeros(dimy*n,dimy*length(theta0)); %blank for derivatives (a-d)
psp_mat=nsp_mat; %blank for derivatives (a+d)

%dif_para=theta0*1e-7; %differentiation step size
dif_para=theta*1e-7; %differentiation step size (when computing curves)

for j=1:length(theta0);

%perturb one parameter at a time
npara=theta0;
ppara=theta0;
npara(j)=theta0(j)-dif_para(j);
ppara(j)=theta0(j)+dif_para(j);



%recompute the spectral density matrices)
[nT1,nTC,nTEPS,nTETA,nRC] = dsgesolv(npara,1);
[pT1,pTC,pTEPS,pTETA,pRC] = dsgesolv(ppara,1);


if nRC==[1;1]; % determinacy 

   nRR = [nTEPS,zeros(nstate,1)];
   nQQ=createcov_as(npara); %covariance matrix, zeros for indeterminacy param's
   
elseif nRC==[1;0]; % indeterminacy 
   nTETA=rref(nTETA').'; %reduced column echelon form for nTETA
   nRR = [nTEPS,nTETA]; %note - to conform with Gauss/LS gensys
   nQQ=createcov_as(npara); %augmented covariance matrix
else
   G=[];
   disp('Nonexistence or numerical problems - cannot compute derivative')
   return
   
end


if pRC==[1;1]; % determinacy 

   pRR = [pTEPS,zeros(nstate,1)];
   pQQ=createcov_as(ppara); %covariance matrix, zeros for indeterminacy param's
   
elseif pRC==[1;0]; % indeterminacy 
   pTETA=rref(pTETA').'; %reduced column echelon form for nTETA
   pRR = [pTEPS,pTETA]; %note - to conform with Gauss/LS gensys
   pQQ=createcov_as(ppara); %augmented covariance matrix
else
   G=[];
   disp('Nonexistence or numerical problems - cannot compute derivative')
   return
   
end



for i=1:n;
exe=exp(-1i*x(i));
%A(2,2)=(1-exe);
nmat1=(eye(nstate)-nT1*exe)\eye(nstate);
nmat2=nmat1';

pmat1=(eye(nstate)-pT1*exe)\eye(nstate);
pmat2=pmat1';

nsp_mat(((i-1)*dimy+1):(i*dimy),((j-1)*dimy+1):(j*dimy))=A*nmat1*nRR*nQQ*nRR'*nmat2*A'/(2*pi); %spectra at perturbed theta0
psp_mat(((i-1)*dimy+1):(i*dimy),((j-1)*dimy+1):(j*dimy))=A*pmat1*pRR*pQQ*pRR'*pmat2*A'/(2*pi); %spectra at perturbed theta0
%A(2,2)=1;
end
end

%now construct the G matrix, note that the jkth component is simply the derivative with respect to j and k
%a symmetric difference quotient method is used

Gmat=zeros(length(theta0),length(theta0)); %blank

for j=1:length(theta0)

for k=1:length(theta0)
aj=(psp_mat(:,((j-1)*dimy+1):(j*dimy))-nsp_mat(:,((j-1)*dimy+1):(j*dimy)))/(2*dif_para(j));
ak=(psp_mat(:,((k-1)*dimy+1):(k*dimy))-nsp_mat(:,((k-1)*dimy+1):(k*dimy)))/(2*dif_para(k));

for freq=1:n;
Gmat(j,k)=Gmat(j,k)+wg(freq)*sum(diag(aj(((freq-1)*dimy+1):(freq*dimy),:)*ak(((freq-1)*dimy+1):(freq*dimy),:)));

end

end

end
%Gmat=2*pi*Gmat/n;
G=real(Gmat); 
% if RC==[1;1]
%     G=G(1:11,1:11); %trim zeros for determinacy case
% end

end