function G = gmatls(theta0,n)

%Function to compute local identification condition for the LS(2004) model at
%a point theta0 under both determinacy and indeterminacy. 
%Inputs: theta0 - vector of model parameters in the order:
%[tau beta kappa psi1 psi2 rhoR rhog rhoz sigR sigg sigz,m_eta_r,m_eta_g,m_eta_a,sig_eta]';
% n - number of points for evaluating the integral

%Observables order: 1) inflation, 2) output, 3) interest rate

%uses auxiliary files createcov_ls.m, lubiksolv.m, gensys_mod.m, svdrr.m

%% Model solution
%para=partranls(theta0,2); %transofrm parameters for model solution
[TT,TC,TEPS,TETA,RC] = lubiksolv(theta0,1); 
nstate = size(TT,2); %number of state variables


if RC==[1;1] %if determinacy
    RR = [TEPS,zeros(nstate,1)];
    
    cgy=theta0(12)*theta0(10)*theta0(11); %cov(g,z) 
    QQ=createcov_ls(theta0); %covariance matrix, zeros for indeterminacy param's
    
elseif RC==[1;0] %if indeterminacy
    TETA=rref(TETA').'; %reduced column echelon form for TETA
    RR = [TEPS,TETA]; %note - to conform with Gauss/LS gensys
    QQ=createcov_ls(theta0); %augmented covariance matrix
else
    G=[];
    disp('Nonexistence or numerical problems')
    return
end
%% Spectrum computation
%set up selection matrix
A=[4	0	0	0	0	0	0;
   0	1	0	0	0	0	0;
   0	0	4	0	0	0	0]; %selection matrix A

%the integral [-pi,pi] is approximated using n points
%n=10000;
omi=2*pi*(-(n/2-1):1:(n/2))'/n; %form vector of Fourier frequencies for approximation of the integral in G
dimy=size(A,1); %number of observables
sp_mat=zeros(dimy*n,dimy); %blank for spectra at all frequencies in omi
for i=1:n
exe=exp(-1i*omi(i));
mat1=(eye(nstate)-TT*exe)\eye(nstate); %inv(1-T1L)
mat2=mat1';  %note that ' gives the conjugate transpose

sp_mat(((i-1)*dimy+1):(i*dimy),:)=A*mat1*RR*QQ*RR'*mat2*A'/(2*pi);

end
%% Numerical derivative
nsp_mat=zeros(dimy*n,dimy*length(theta0)); %blank for derivatives
dif_para=theta0*1e-7; %differentiation step size

for j=1:length(theta0);

%perturb one parameter at a time
npara=theta0;
npara(j)=theta0(j)-dif_para(j);
%ntheta=partranls(npara,2); %transofrm parameters for model solution

%recompute the spectral density matrix)
[nT1,nTC,nTEPS,nTETA,nRC] = lubiksolv(npara,1);

if nRC==[1;1]; % determinacy 

   nRR = [nTEPS,zeros(nstate,1)];
   
   nQQ=createcov_ls(npara); %covariance matrix, zeros for indeterminacy param's
   
elseif nRC==[1;0]; % indeterminacy 
   nTETA=rref(nTETA').'; %reduced column echelon form for nTETA
   nRR = [nTEPS,nTETA]; %note - to conform with Gauss/LS gensys
   nQQ=createcov_ls(npara); %augmented covariance matrix
else
   G=[];
   disp('Nonexistence or numerical problems - cannot compute derivative')
   return
   
end



for i=1:n;
exe=exp(-1i*omi(i));
nmat1=(eye(nstate)-nT1*exe)\eye(nstate);
nmat2=nmat1';

nsp_mat(((i-1)*dimy+1):(i*dimy),((j-1)*dimy+1):(j*dimy))=A*nmat1*nRR*nQQ*nRR'*nmat2*A'/(2*pi); %spectra at perturbed theta0
end
end

%now construct the G matrix, note that the jkth component is simply the derivative with respect to j and k
%a simple two-point method is used

Gmat=zeros(length(theta0),length(theta0)); %blank

for j=1:length(theta0)

for k=1:length(theta0)
aj=(sp_mat-nsp_mat(:,((j-1)*dimy+1):(j*dimy)))/dif_para(j);
ak=(sp_mat-nsp_mat(:,((k-1)*dimy+1):(k*dimy)))/dif_para(k);

for freq=1:n;
Gmat(j,k)=Gmat(j,k)+sum(diag(aj(((freq-1)*dimy+1):(freq*dimy),:)*ak(((freq-1)*dimy+1):(freq*dimy),:)));

end

end

end
Gmat=2*pi*Gmat/n;
G=real(Gmat); 
% if RC==[1;1]
%     G=G(1:12,1:12); %trim zeros for determinacy case
% end

end