%******************************************************************************
%*      Solve LS (2004) new Keynesian Monetary Policy DSGE using GENSYS
%*
%*      
%**************************************************************************/
function [T1,TC,T0,TETA,RC] = lubiksolv(para,msel)

%Model parameters
%paranames = theta=[tau beta kappa psi1 psi2 rhoR rhog rhoz sigR sigg sigz rhogz m_eta_r m_eta_g m_eta_z sig_eta];

% note only parameters 1-8 affect the dynamic solution here, the rest enter the
% covariance matrix.

%also note: in this model tau is the inverse of "tau" in AS(2007)

%msel selects the model specification:
% 1 - current inflation and output gap in the Taylor rule
% 2 - future inflation and output gap in the Taylor rule 
% 3 - current inflation and output growth in the Taylor rule
%********************************************************************
%**      assign names to parameters
%********************************************************************
tau        = para(1);
kap        = para(3);
psi1       = para(4);
psi2       = para(5);
rhoR       = para(6);
rhog       = para(7);
rhoz       = para(8);

%** additional parameters **
bet =para(2);
%bet=0.997; %uncomment for identification analysis with fixed beta
%********************************************************************
%**      matrices of canonical system
%********************************************************************

%** equation indices **
eq_euler = 1;  % Euler equation **
eq_p     = 2;  % Phillip's Curve **
eq_mp    = 3;  % Monetary policy rule **
eq_g     = 4;  % g process **
eq_z     = 5;  % z process **
eq_Epi   = 6;  % E(pi) **
eq_Ey    = 7;  % E(y) **


%** variable indices **
v_pi  = 1;
v_y   = 2;
v_R   = 3;
v_g   = 4;
v_z   = 5;
v_Epi = 6;
v_Ey  = 7;

%** shock indices (neps) **
e_R = 1;
e_g = 2;
e_z = 3;

%** expectation error indices (neta) **
n_pi = 1;
n_y  = 2;

%** summary **
neq  = 7;
neps = 3;
neta = 2;

%** initialize matrices **
GAM0 = zeros(neq,neq);
GAM1 = zeros(neq,neq);
   C = zeros(neq,1);
 PSI = zeros(neq,neps);
 PPI = zeros(neq,neta);

%**********************************************************
%**      1. Euler Equation
%**********************************************************
GAM0(eq_euler,v_y)   = -1;
GAM0(eq_euler,v_Ey)  =  1;
GAM0(eq_euler,v_R)   = -tau;
GAM0(eq_euler,v_Epi) =  tau;
GAM0(eq_euler,v_g)   =  1;

%**********************************************************
%**      2. Phillip's Curve
%**********************************************************
GAM0(eq_p,v_pi)  = -1;
GAM0(eq_p,v_Epi) =  bet;
GAM0(eq_p,v_y)   =  kap;
GAM0(eq_p,v_z)   = -kap;

%**********************************************************
%**      3. Monetary policy rule
%**********************************************************


%   
  if msel==1
  GAM0(eq_mp,v_R)  =   1;
  GAM0(eq_mp,v_pi) = -(1-rhoR)*psi1; %current inflation
  GAM0(eq_mp,v_y)  = -(1-rhoR)*psi2;
  GAM0(eq_mp,v_z)  = (1-rhoR)*psi2;
  GAM1(eq_mp,v_R)  =   rhoR;
  PSI(eq_mp,e_R)  =   1;
  elseif msel==2
      GAM0(eq_mp,v_R)  =   1;
      GAM0(eq_mp,v_Epi) = -(1-rhoR)*psi1; %future inflation
      GAM0(eq_mp,v_y)  = -(1-rhoR)*psi2;
      GAM0(eq_mp,v_z)  = (1-rhoR)*psi2; 
      GAM1(eq_mp,v_R)  =   rhoR;
      PSI(eq_mp,e_R)  =   1;
  elseif msel==3 %output growth rule
  GAM0(eq_mp,v_R)  =   1;
  GAM0(eq_mp,v_pi) = -(1-rhoR)*psi1; %current inflation
  GAM0(eq_mp,v_y)  = -(1-rhoR)*psi2;
  GAM1(eq_mp,v_y)  = -(1-rhoR)*psi2;
  GAM1(eq_mp,v_R)  =   rhoR;
      PSI(eq_mp,e_R)  =   1;
  end



 



%**********************************************************
%**      Shock process
%**********************************************************
%** g **
GAM0(eq_g,v_g) = 1;
GAM1(eq_g,v_g) = rhog;
 PSI(eq_g,e_g) = 1;

%** z **
GAM0(eq_z,v_z) = 1;
GAM1(eq_z,v_z) = rhoz;
 PSI(eq_z,e_z) = 1;

%**********************************************************
%**      Expectation error
%**********************************************************
%** E[pi] **
GAM0(eq_Epi,v_pi)  = 1;
GAM1(eq_Epi,v_Epi) = 1;
 PPI(eq_Epi,n_pi)  = 1;
%** E[y] **
GAM0(eq_Ey,v_y)  = 1;
GAM1(eq_Ey,v_Ey) = 1;
 PPI(eq_Ey,n_y)  = 1;


%********************************************************************
%**      QZ(generalized Schur) decomposition by GENSYS
%********************************************************************
 %[T1,TC,T0,TY,M,TZ,TETA,GEV,RC]=gensyss(GAM0, GAM1, C, PSI, PPI, 1); % use Gauss code translation with ordqz
  [T1,TC,T0,TY,M,TZ,TETA,GEV,RC]=gensys_mod(GAM0, GAM1, C, PSI, PPI, 1); %use Sims-based code, also uses ordqz
  
  if RC(1)==1 && RC(2)==0 %if indeterminacy, compute psi1 on the boundary (eq. (31) in LS(2004))
       psi1star = 1 - bet*psi2*(1/bet-1)/kap-(1E-8);
       GAM0star = GAM0;
       %now replace psi1 by psi1star in the GAM0star as follows:
       if msel==1 || msel==3 %if current inflation
          GAM0star(eq_mp, v_pi) = -psi1star*(1-rhoR);
       elseif msel==2 %if future inflation
          GAM0star(eq_mp, v_Epi) = -psi1star*(1-rhoR);
           
       end
       
   %/* Evaluate Model under determinacy / indeterminacy by shifting cutoff */
   [T1star,TCstar,T0starind,TYstar,Mstar,TZstar,TETAstarind,GEVstar,RCstarind] = gensys_mod(GAM0star,GAM1,C,PSI,PPI,1+1E-6);
   [T1star,TCstar,T0stardet,TYstar,Mstar,TZstar,TETAstardet,GEVstar,RCstardet]  = gensys_mod(GAM0star,GAM1,C,PSI,PPI,1-1E-6);
      
   META = inv(TETA'*TETA)*TETA'*(T0stardet-T0); %see display (30) in LS(2004)
   T0 = T0 + TETA*META;
  end
end
